@@ -2,16 +2,18 @@
 #'
 #' @description
 #'
-#' This function estimates a local log-Gaussian Cox process (LGCP), following the
-#' *locally weighted minimum contrast* procedure introduced in
-#' D'Angelo et al. (2023).
+#' This function estimates a log-Gaussian Cox process (LGCP), following the
+#' **joint minimum contrast** procedure introduced in Siino et al. (2018)
+#' .
 #'
 #' Three covariances are available: separable exponential,
 #' Gneiting, and De Iaco-Cesare.
 #'
-#' If both \code{first} and \code{second} arguments are set to \code{global}, a log-Gaussian
-#' Cox process is fitted by means of the *joint minimum contrast* procedure proposed in
-#' Siino et al. (2018).
+#' If the \code{first} and \code{second} arguments are set to \code{local}, a local
+#'  log-Gaussian
+#' Cox process is fitted by means of the ** locally weighted minimum contrast**
+#'  procedure proposed in
+#' D'Angelo et al. (2023).
 #'
 #' @details
 #'
@@ -21,42 +23,21 @@
 #' \Lambda(\textbf{u},t)=\lambda(\textbf{u},t)\exp(S(\textbf{u},t)).
 #' }
 #'
-#' Following Siino et al. (2018), the second-order
-#' parameters \eqn{\boldsymbol{\psi}} are found by minimising
-#' \deqn{  M_J\{ \boldsymbol{\psi}\}=\int_{h_0}^{h_{max}} \int_{r_0}^{r_{max}} \phi(r,h) \{\nu[\hat{J}(r,h)]-\nu[J(r,h;\boldsymbol{\psi})]\}^2 \text{d}r \text{d}h,
-#' } where \eqn{\phi(r, h)} is a weight that depends on the space-time
-#' distance and \eqn{\nu} is a transformation function.
 #'
-#' They suggest \eqn{\phi(r,h)=1} and \eqn{\nu} as
-#' the identity function, while \eqn{r_{max}} and \eqn{h_{max}} are selected as 1/4
-#' of the maximum observable spatial and temporal distances.
-#'
-#' Following D'Angelo et al. (2023), we can fit a localised version of the LGCP,
-#' that is,  obtain a
-#' vector of parameters \eqn{\boldsymbol{\psi}_i} for each point \eqn{i}, by
-#' minimising \deqn{
-#'   M_{J,i}\{ \boldsymbol{\psi}_i \}=\int_{h_0}^{h_{max}}\int_{r_0}^{r_{max}}
-#'    \phi(r,h) \{ \nu[\bar{J}_i(r,h)]-\nu[J(r,h;\boldsymbol{\psi})]\}^2 \text{d}r \text{d}h,
-#' } where \eqn{ \bar{J}_i(r,h)= \frac{\sum_{i=1}^{n}\hat{J}_i(r,h)w_i}{\sum_{i=1}^{n}w_i}} is the average of the local functions
-#' \eqn{\hat{J}_i(r,h)}, weighted by some point-wise kernel estimates.
-#'
-#'   In particular, we consider \eqn{\hat{J}_i(\cdot)} as the local
-#' spatio-temporal pair correlation function (Gabriel et al, 2013) documented in \link{LISTAhat}.
-#'
-#'
-#' @param X A stp object
+#' @param X A \code{stp} object
 #' @param formula An object of class \code{formula}: a symbolic description of the first-order intensity to be fitted.
 #' The current version only supports formulas depending on the spatial and temporal coordinates:
-#' \code{x}, \code{y}, \code{t}.
+#' \code{x}, \code{y}, \code{t}. Default to \code{formula = ~ 1} which provides an homogeneous
+#' first-order intensity.
 #' @param verbose Default to TRUE
 #' @param cov Covariance function to be fitted for the second-order intensity function.
 #' Default to \code{separable}. Other options are \code{gneiting} and \code{iaco-cesare}".
 #' @param first Character string indicating whether to fit a first-order intensity function
 #'  with global or local parameters:
-#'  either \code{local} (default) or \code{global}.
+#'  either \code{global} (default) or \code{local}.
 #' @param second Character string indicating whether to fit a second-order intensity function
 #'  with global or local parameters:
-#'  either \code{local} (default) or \code{global}.
+#'  either \code{global} (default) or \code{local}.
 #' @param mult The multiplicand of the number of data points,
 #'  for setting the number of dummy
 #' points to generate for the quadrature scheme
@@ -65,7 +46,7 @@
 #' In any of those cases, the well-supported rule-of-thumb for choosing the
 #' bandwidth of a Gaussian kernel density estimator is employed.
 #' If \code{hs = "global"} (default), a fixed bandwidth is selected.
-#' If \code{hs = "local"}, an individual badwidth is selected for each point in the
+#' If \code{hs = "local"}, an individual bandwidth is selected for each point in the
 #' pattern  \code{X}.
 #' @param npx0 A positive integer representing the spatial distance to np-th closest event.
 #' Used in the computation of the local bandwidth.
@@ -74,13 +55,15 @@
 #' Used in the computation of the local bandwidth.
 #' Suitable values are in the range from 10 (default) to 100.
 #' @param itnmax Maximum number of iterations to run in the optimization procedure
-#' for the estiamtion of the second-order intensity parameters.
+#' for the estimation of the second-order intensity parameters.
+#' @param min_vals Minimum values of the optimization procedure for the minimum contrast.
+#' @param max_vals Maximum values of the optimization procedure for the minimum contrast.
 #'
 #' @return A list of the class \code{stlgcppm}, containing
 #' \describe{
 #' \item{\code{IntCoefs}}{The fitted coefficients of the first-order intensity function}
 #' \item{\code{CovCoefs}}{The fitted coefficients of the second-order intensity function}
-#' \item{\code{X}}{The stp objectt provided as input}
+#' \item{\code{X}}{The stp object provided as input}
 #' \item{\code{formula}}{The formula provided as input}
 #' \item{\code{cov}}{A string with the chosen covariance type}
 #' \item{\code{l}}{Fitted first-order intensity}
@@ -105,37 +88,11 @@
 #' @examples
 #'
 #' \dontrun{
-#' # Example with complex seismic point pattern
-#' data("greececatalog")
-#'
-#' ##
-#'
-#' # If both first and second arguments are set to "global", a log-Gaussian
-#' # Cox process is fitted by means of the joint minimum contrast.
-#'
-#' lgcp1 <- stlgcppm(greececatalog, formula = ~ 1, first = "global", second = "global")
-#'
-#'  ##
-#'
-#'  # If first = "local", local parameters for the first-order intensity are
-#'  # provided. In this case, the summary function contains information on
-#'  # their distributions.
-#'
-#'  lgcp2 <- stlgcppm(greececatalog, formula = ~ x, first = "local", second = "global")
-#'
-#'  ##
-#'
-#' # Finally, if second = "local" (default option), the model with local
-#'  #parameters for the covariance is fitted.
-#'
-#'  lgcp3 <- stlgcppm(greececatalog, formula = ~ 1, first = "global", second = "local")
-#'
-#'  ##
-#'
-#' #  It is also possible to fit local parameters for both the first and
-#' #  second-order intensity, but we abstain from addressing this case here.
-#'
-#'
+#' 
+#' catsub <- stp(greececatalog$df[1:100, ])
+#' 
+#' lgcp1 <- stlgcppm(catsub)
+#' 
 #'}
 #'
 #'
@@ -151,85 +108,96 @@
 #'
 #' Siino, M., Adelfio, G., and Mateu, J. (2018). Joint second-order parameter estimation for spatio-temporal log-Gaussian Cox processes. Stochastic environmental research and risk assessment, 32(12), 3525-3539.
 #'
-stlgcppm <- function(X, formula, verbose = TRUE, cov = "separable",
-                   first = "local", second = "local", mult = 4,
-                   hs = "global", npx0 = 10, npt0 = 10, itnmax = 100){
+stlgcppm <- function(X, formula = ~ 1, verbose = TRUE,
+                     cov = c("separable", "gneiting", "iaco-cesare"),
+                     first = c("global", "local"), second = c("global", "local"), mult = 4,
+                     hs = c("global", "local"), npx0 = 10, npt0 = 10, itnmax = 100,
+                     min_vals = NULL, max_vals = NULL){
+  
+  if (!inherits(X, c("stp"))) stop("X should be from class stp")
+  
   time1 <- Sys.time()
+  
+  cov <- match.arg(cov)
+  first <- match.arg(first)
+  second <- match.arg(second)
+  hs <- match.arg(hs)
+  
   nX <- nrow(X$df)
-
+  
   x <- X$df$x
   y <- X$df$y
   t <- X$df$t
-
+  
   s.region <- splancs::sbox(cbind(x, y), xfrac = 0.01, yfrac = 0.01)
   xr = range(t, na.rm = TRUE)
   xw = diff(xr)
   t.region <- c(xr[1] - 0.01 * xw, xr[2] + 0.01 * xw)
-
+  
   HomLambda <- nX
-
+  
   rho <- mult * HomLambda
-
+  
   dummy_points <- rstpp(lambda = rho, nsim = 1, seed = 2, verbose = F,
-                               minX = s.region[1, 1], maxX = s.region[2, 1],
-                               minY = s.region[1, 2], maxY = s.region[3, 2],
-                               minT = t.region[1], maxT = t.region[2])$df
-
+                        minX = s.region[1, 1], maxX = s.region[2, 1],
+                        minY = s.region[1, 2], maxY = s.region[3, 2],
+                        minT = t.region[1], maxT = t.region[2])$df
+  
   quad_p <- rbind(X$df, dummy_points)
-
+  
   z <- c(rep(1, nX), rep(0, dim(dummy_points)[1]))
-
+  
   xx   <- quad_p[, 1]
   xy   <- quad_p[, 2]
   xt   <- quad_p[, 3]
   win <- spatstat.geom::box3(xrange = range(xx), yrange = range(xy), zrange = range(xt))
-
+  
   ncube <- .default.ncube(quad_p)
   length(ncube) == 1
   ncube <- rep.int(ncube, 3)
   nx <- ncube[1]
   ny <- ncube[2]
   nt <- ncube[3]
-
+  
   nxyt <- nx * ny * nt
   cubevolume <-  spatstat.geom::volume(win) / nxyt
   volumes <- rep.int(cubevolume, nxyt)
-
+  
   id <- .grid.index(xx, xy, xt,
                     win$xrange, win$yrange, win$zrange, nx, ny, nt)$index
-
+  
   w <- .counting.weights(id, volumes)
-
+  
   y_resp <- z / w
-
+  
   dati.modello <- cbind(y_resp, w, quad_p[, 1], quad_p[, 2], quad_p[, 3])
   colnames(dati.modello) <- c("y_resp", "w", "x", "y", "t")
-
+  
   dati.modello <- as.data.frame(dati.modello)
-
+  
   suppressWarnings(mod_global <- try(glm(as.formula(paste("y_resp", paste(formula, collapse = " "), sep = " ")),
-                        weights = w, family = poisson, data = dati.modello), silent = T))
+                                         weights = w, family = poisson, data = dati.modello), silent = T))
   res_global <- mod_global$coefficients
-  pred_global <- predict(mod_global, newdata = dati.modello[1:nX, ])
-
+  pred_global <- exp(predict(mod_global, newdata = dati.modello[1:nX, ]))
+  
   nU <- dim(quad_p)[1]
   xx <- quad_p[, 1]
   xy <- quad_p[, 2]
   xt <- quad_p[, 3]
-
+  
   h_x <- MASS::bandwidth.nrd(x)
   h_y <- MASS::bandwidth.nrd(y)
   h_t <- MASS::bandwidth.nrd(t)
-
+  
   if(first == "local"){
-
+    
     if(hs == "local"){
       h_x <- h_y <- kde2d.new.var(x, y, gx = x, gy = y, np = npx0)$hx
       h_t <- density.new.var(t, gx = t, np = npt0)$h
     }
-
+    
     localwt <- matrix(NA, nrow = nX, ncol = nU)
-
+    
     if(verbose) cat(paste("\n", "Computing Kernel Densities to the", nX,
                           "points", "\n", "\n"))
     for(j in 1:nX) {
@@ -244,9 +212,9 @@ stlgcppm <- function(X, formula, verbose = TRUE, cov = "separable",
                                                                 sd = h_y) * dnorm(xt - t[j],
                                                                                   sd = h_t))
     }
-
+    
     a_s <- localwt * w
-
+    
     res_local <- matrix(NA, nrow = nX, ncol = length(mod_global$coefficients))
     pred_local <- vector(length = nX)
     if(verbose) cat(paste("\n", "Fitting local model to the", nX,
@@ -254,37 +222,37 @@ stlgcppm <- function(X, formula, verbose = TRUE, cov = "separable",
     for(i in 1:nX){
       progressreport(i, nX)
       suppressWarnings(mod_local <- try(glm(as.formula(paste("y_resp", paste(formula, collapse = " "), sep = " ")),
-                           weights = a_s[i, ], family = poisson, data = dati.modello), silent = T))
+                                            weights = a_s[i, ], family = poisson, data = dati.modello), silent = T))
       res_local[i, ] <- mod_local$coefficients
-      pred_local[i] <- predict(mod_local, newdata = dati.modello[i, ])
+      pred_local[i] <- exp(predict(mod_local, newdata = dati.modello[i, ]))
     }
-
-    }
-
+    
+  }
+  
   # covariance
-
+  
   tlim <- max(t)
   cube <- (range(x) - min(x))[2]
-
+  
   lgrid_s <- 25
   rsup_t <- tlim / 4
-
+  
   dt <- dist(t)
   bw_vector_t <- KernSmooth::dpik(dt, kernel = "box", range.x = c(min(dt), max(dt)))
-
+  
   rmin_vector_t2 <- (bw_vector_t * ((1 + lgrid_s) / lgrid_s))
-
+  
   vals_t <- cbind(rmin_vector_t2, bw_vector_t)
-
+  
   rsup_s <- cube / 4
-
+  
   ds <- dist(cbind(x, y))
   bw2_vector <- KernSmooth::dpik(ds, kernel = "epanech", range.x = c(min(ds), max(ds)))
-
+  
   rmin3_vector <- bw2_vector * ((1 + lgrid_s) / lgrid_s)
-
+  
   vals_s <- cbind(rmin3_vector, bw2_vector)
-
+  
   s.region <- matrix(c(range(x)[1] - 0.05,
                        range(x)[2] + 0.05,
                        range(x)[2] + 0.05,
@@ -293,43 +261,42 @@ stlgcppm <- function(X, formula, verbose = TRUE, cov = "separable",
                        range(y)[1] - 0.05,
                        range(y)[2] + 0.05,
                        range(y)[2] + 0.05), ncol = 2)
-
+  
   t.region <- c(min(t) - 1, tlim + 1)
-
-
+  
+  
   u <- seq(vals_s[1], rsup_s, len = lgrid_s)
   v <- seq(vals_t[1], rsup_t, len = lgrid_s)
   us0 <- vals_s[2]
   vt0 <- vals_t[2]
-
+  
   g0 <- if(first == "global"){
-    stpp::LISTAhat(xyt = as.stpp(X), lambda = exp(pred_global), s.region = s.region, t.region = t.region,
-             dist = u, times = v, ks = "epanech", hs = us0, kt = "box",
-             ht = vt0)
+    stpp::LISTAhat(xyt = as.stpp(X), lambda = pred_global, s.region = s.region, t.region = t.region,
+                   dist = u, times = v, ks = "epanech", hs = us0, kt = "box",
+                   ht = vt0)
   } else {
-    stpp::LISTAhat(xyt = as.stpp(X), lambda = exp(pred_local), s.region = s.region, t.region = t.region,
-             dist = u, times = v, ks = "epanech", hs = us0, kt = "box",
-             ht = vt0)
+    stpp::LISTAhat(xyt = as.stpp(X), lambda = pred_local, s.region = s.region, t.region = t.region,
+                   dist = u, times = v, ks = "epanech", hs = us0, kt = "box",
+                   ht = vt0)
   }
-
+  
   max_dist <- max(dist(s.region))
   max_dist_t <- max(diff(t.region))
-
+  
   nonpar_g_st_finite <- is.finite(g0$list.LISTA)
   g0$list.LISTA[!nonpar_g_st_finite] <- 0
-
+  
   if(cov == "separable"){
-
-    min_vals <- c(0.01, 0.001, 20)
-
-    max_vals <-  c(50, 10, 1500)
-
+    
+    if(is.null(min_vals)) min_vals <- c(0.01, 0.001, 0.01)#c(0.01, 0.001, 20)
+    if(is.null(max_vals)) max_vals <-  c(50, 10, 1500)
+    
     start_par <- c(log(nX) / 2, max_dist / 100, max_dist_t / 100)
-
+    
     if(second == "global"){
-
+      
       MINCON.EXP_EXP <- c(NA, NA, NA)
-
+      
       MINCON.EXP_EXP <- optimx::optimx(par = start_par,
                                        fn = g.sep_st_exp_exp2,
                                        useq = g0$dist,
@@ -340,34 +307,34 @@ stlgcppm <- function(X, formula, verbose = TRUE, cov = "separable",
                                        lower = min_vals,
                                        upper = max_vals,
                                        itnmax = itnmax)
-
+      
       res <- as.numeric(as.vector(MINCON.EXP_EXP[1 : 3]))
-
+      
     } else {
-
+      
       res <- matrix(NA, nrow = nX, ncol = 3)
-
+      
       if(verbose) cat(paste("\n", "Fitting local Separable covariance to the", nX,
                             "points", "\n", "\n"))
-
+      
       for(id in 1:nX){
-
+        
         spatstat.geom::progressreport(id, nX)
-
+        
         MINCON.EXP_EXP <- c(NA, NA, NA)
-
+        
         wi <- dnorm(x - x[id], sd = h_x) * dnorm(y - y[id], sd = h_y) * dnorm(t - t[id], sd = h_t)
-
+        
         numer0 <- sapply(1:nX, function(x) g0$list.LISTA[, , x] * wi[x], simplify = "array")
-
+        
         numer <- apply(numer0, 1:2, sum)
-
+        
         denom0 <- sapply(1:nX, function(x) nonpar_g_st_finite[, , x] * wi[x], simplify = "array")
-
+        
         denom <- apply(denom0, 1:2, sum)
-
+        
         avg_lista <- numer / denom
-
+        
         MINCON.EXP_EXP <- optimx::optimx(par = start_par,
                                          fn = g.sep_st_exp_exp2,
                                          useq = g0$dist,
@@ -378,26 +345,26 @@ stlgcppm <- function(X, formula, verbose = TRUE, cov = "separable",
                                          lower = min_vals,
                                          upper = max_vals,
                                          itnmax = itnmax)
-
-
+        
+        
         res[id, ] <- as.numeric(as.vector(MINCON.EXP_EXP[1 : 3]))
       }
-
+      
     }
-
+    
   }
-
+  
   if(cov == "gneiting") {
-
-    min_vals <- c(0.01, 0.01, 0.01, 0.01, 0.01, 0.01)
-    max_vals <- c(nX, max_dist * 250, max_dist_t * 250, 2, 2, 2)
-
+    
+    if(is.null(min_vals)) min_vals <- c(0.01, 0.01, 0.01, 0.01, 0.01, 0.01)
+    if(is.null(max_vals)) max_vals <- c(nX, max_dist * 250, max_dist_t * 250, 2, 2, 2)
+    
     start_par <- c(log(nX) / 2, max_dist, max_dist_t, 1, 1, 1)
-
+    
     if(second == "global"){
-
+      
       MINCON_GN <- c(NA, NA, NA, NA, NA, NA)
-
+      
       MINCON_GN <- optimx::optimx(par = start_par,
                                   fn = g_st,
                                   useq = g0$dist,
@@ -409,34 +376,34 @@ stlgcppm <- function(X, formula, verbose = TRUE, cov = "separable",
                                   lower = min_vals,
                                   upper = max_vals,
                                   itnmax = itnmax)
-
+      
       res <- as.numeric(as.vector(MINCON_GN[1 : 6]))
-
+      
     } else {
-
+      
       res <- matrix(NA, nrow = nX, ncol = 6)
-
+      
       if(verbose) cat(paste("\n", "Fitting local Gneiting covariance to the", nX,
                             "points", "\n", "\n"))
-
+      
       for(id in 1:nX){
-
+        
         spatstat.geom::progressreport(id, nX)
-
+        
         MINCON_GN <- c(NA, NA, NA, NA, NA, NA)
-
+        
         wi <- dnorm(x - x[id], sd = h_x) * dnorm(y - y[id], sd = h_y) * dnorm(t - t[id], sd = h_t)
-
+        
         numer0 <- sapply(1:nX, function(x) g0$list.LISTA[, , x] * wi[x], simplify = "array")
-
+        
         numer <- apply(numer0, 1:2, sum)
-
+        
         denom0 <- sapply(1:nX, function(x) nonpar_g_st_finite[, , x] * wi[x], simplify = "array")
-
+        
         denom <- apply(denom0, 1:2, sum)
-
+        
         avg_lista <- numer / denom
-
+        
         MINCON_GN <- optimx::optimx(par = start_par,
                                     fn = g_st,
                                     useq = g0$dist,
@@ -448,23 +415,23 @@ stlgcppm <- function(X, formula, verbose = TRUE, cov = "separable",
                                     lower = min_vals,
                                     upper = max_vals,
                                     itnmax = itnmax)
-
+        
         res[id, ] <- as.numeric(as.vector(MINCON_GN[1 : 6]))
       }
     }
   }
-
+  
   if(cov == "iaco-cesare") {
-
-    min_vals <- c(0.01, 0.01, 0.01, 0.01, 0.01, 1.5)
-    max_vals <- c(nX, max_dist * 250, max_dist_t * 250, 2, 2, 15)
-
+    
+    if(is.null(min_vals)) min_vals <- c(0.01, 0.01, 0.01, 0.01, 0.01, 1.5)
+    if(is.null(max_vals)) max_vals <- c(nX, max_dist * 250, max_dist_t * 250, 2, 2, 15)
+    
     start_par <- c(log(nX) / 2, max_dist, max_dist_t, 1, 1, 8)
-
+    
     if(second == "global"){
-
+      
       MINCON_IACO <- c(NA, NA, NA, NA, NA, NA)
-
+      
       MINCON_IACO <- optimx::optimx(par = start_par,
                                     fn = g_st_iaco,
                                     useq = g0$dist,
@@ -476,35 +443,35 @@ stlgcppm <- function(X, formula, verbose = TRUE, cov = "separable",
                                     lower = min_vals,
                                     upper = max_vals,
                                     itnmax = itnmax)
-
-
+      
+      
       res <- as.numeric(as.vector(MINCON_IACO[1 : 6]))
-
+      
     } else {
-
+      
       res <- matrix(NA, nrow = nX, ncol = 6)
-
+      
       if(verbose) cat(paste("\n", "Fitting local Iaco-Cesare covariance to the", nX,
                             "points", "\n", "\n"))
-
+      
       for(id in 1:nX){
-
+        
         spatstat.geom::progressreport(id, nX)
-
+        
         MINCON_IACO <- c(NA, NA, NA, NA, NA, NA)
-
+        
         wi <- dnorm(x - x[id], sd = h_x) * dnorm(y - y[id], sd = h_y) * dnorm(t - t[id], sd = h_t)
-
+        
         numer0 <- sapply(1:nX, function(x) g0$list.LISTA[, , x] * wi[x], simplify = "array")
-
+        
         numer <- apply(numer0, 1:2, sum)
-
+        
         denom0 <- sapply(1:nX, function(x) nonpar_g_st_finite[, , x] * wi[x], simplify = "array")
-
+        
         denom <- apply(denom0, 1:2, sum)
-
+        
         avg_lista <- numer / denom
-
+        
         MINCON_IACO <- optimx::optimx(par = start_par,
                                       fn = g_st_iaco,
                                       useq = g0$dist,
@@ -516,47 +483,47 @@ stlgcppm <- function(X, formula, verbose = TRUE, cov = "separable",
                                       lower = min_vals,
                                       upper = max_vals,
                                       itnmax = itnmax)
-
-
+        
+        
         res[id, ] <- as.numeric(as.vector(MINCON_IACO[1 : 6]))
       }
     }
   }
-
+  
   time2 <- Sys.time()
-
+  
   if(second == "local"){
-
+    
     colnames(res) <- switch(cov,
                             "separable" = c("sigma", "alpha", "beta"),
                             "gneiting" = c("sigma", "alpha", "beta", "gamma_s", "gamma_t", "delta"),
                             "iaco-cesare" = c("sigma", "alpha", "beta", "gamma_s", "gamma_t", "delta"))
-
-
+    
+    
     res <- as.data.frame(res)
-
+    
   } else {
-
+    
     names(res) <- switch(cov,
                          "separable" = c("sigma", "alpha", "beta"),
                          "gneiting" = c("sigma", "alpha", "beta", "gamma_s", "gamma_t", "delta"),
                          "iaco-cesare" = c("sigma", "alpha", "beta", "gamma_s", "gamma_t", "delta"))
-
+    
   }
-
+  
   if(inherits(res, "numeric")){
     int2 <- rep(res[1], nX) / 2
   } else {
     int2 <- res$sigma / 2
   }
-
+  
   if(first == "local"){
-
+    
     names(res_global) <- names(mod_global$coefficients)
     res_local <- data.frame(res_local)
     colnames(res_local) <- names(mod_global$coefficients)
-
-
+    
+    
     if(hs == "global"){
       bw <- c(round(h_x, 3), round(h_y, 3), round(h_t, 3))
       names(bw) <- c("h_x", "h_y", "h_t")
@@ -564,10 +531,10 @@ stlgcppm <- function(X, formula, verbose = TRUE, cov = "separable",
       bw <- cbind(round(h_x, 3), round(h_y, 3), round(h_t, 3))
       colnames(bw) <- c("h_x", "h_y", "h_t")
     }
-
+    
     quad_p <- rbind(X$df)
     quad_p <- as.data.frame(quad_p)
-
+    
     list.obj <- list(IntCoefs = res_local,
                      CovCoefs = res,
                      X = X,
@@ -579,8 +546,8 @@ stlgcppm <- function(X, formula, verbose = TRUE, cov = "separable",
                      newdata = dati.modello[1:nX, ],
                      time = paste0(round(as.numeric(difftime(time1 = time2, time2 = time1, units = "mins")), 3), " minutes"))
   } else {
-
-
+    
+    
     list.obj <- list(IntCoefs = res_global,
                      CovCoefs = res,
                      X = X,
@@ -591,15 +558,15 @@ stlgcppm <- function(X, formula, verbose = TRUE, cov = "separable",
                      mod_global = mod_global,
                      newdata = dati.modello[1:nX, ],
                      time = paste0(round(as.numeric(difftime(time1 = time2, time2 = time1, units = "mins")), 3), " minutes"))
-
+    
   }
-
-
-
-
+  
+  
+  
+  
   class(list.obj) <- "stlgcppm"
   return(list.obj)
-
+  
 }
 
 
@@ -612,3 +579,5 @@ stlgcppm <- function(X, formula, verbose = TRUE, cov = "separable",
 
 
 
+
+
