@@ -12,39 +12,17 @@
 #'
 #' @seealso
 #' \link{stppm}, \link{print.stppm},
-#' \link{plot.stppm}, \link{coef.stppm}
+#' \link{plot.stppm}
 #'
 #'
 #' @examples
 #' \dontrun{
-#' ## Homogeneous
-#' set.seed(2)
-#' ph <- rstpp(lambda = 200, nsim = 1, seed = 2, verbose = T)
-#' hom1 <- stppm(ph, formula = ~ 1)
 #'
-#' summary(hom1)
-#'
-#' # Homogeneous Poisson process
-#' # with Intensity: 202.093
-#' #
-#' # Estimated coefficients:
-#' #   (Intercept)
-#' # 5.309
-#'
-#' ## Inhomogeneous
 #' pin <- rstpp(lambda = function(x, y, t, a) {exp(a[1] + a[2]*x)}, par = c(2, 6),
-#' nsim = 1, seed = 2, verbose = T)
+#'              seed = 2)
 #' inh1 <- stppm(pin, formula = ~ x)
-#'
+#' 
 #' summary(inh1)
-#'
-#' # Inhomogeneous Poisson process
-#' # with Trend: ~x
-#' #
-#' # Estimated coefficients:
-#' #   (Intercept)           x
-#' # 2.180       5.783
-#' #
 #'}
 #'
 #'
@@ -54,7 +32,8 @@
 #'
 #'
 summary.stppm <- function(object, ...){
-  if(!any(class(object) == "stppm")) stop("class(object) must be stppm")
+  if(!inherits(object, "stppm")) stop("class(object) must be stppm")
+  
   if(length(object$IntCoefs) == 1){
     cat("Homogeneous Poisson process \n")
     cat("with Intensity: ")
